<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();
/**
 * Zoom Online block class
 *
 * @package    block_zoomonline
 * @category   privacy
 * @copyright  2024 Ciaran Mac Donncha
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
/**
 * Handles the upgrade process for the block_zoomonline plugin.
 *
 * This function is called during the plugin upgrade process. It checks the current version of the plugin
 * and performs necessary database schema updates, such as creating or updating the `block_zoomonline_links`
 * table. The table stores links between Zoom meetings and Moodle courses, lecturers, and groups.
 *
 * Upgrade logic:
 * - If the version is less than 2024112702, it ensures the `block_zoomonline_links` table exists.
 * - If the table does not exist, it creates the table with all required fields and a primary key.
 * - If the table already exists, it ensures all fields are present and adds any missing fields.
 *
 * @param int $oldversion The version of the plugin before the upgrade.
 * @return bool True on successful upgrade.
 */
function xmldb_block_zoomonline_upgrade($oldversion) {
    global $DB;

    $dbman = $DB->get_manager(); // Database manager.

    // Ensure the block_zoomonline_links table exists and is correct.
    if ($oldversion < 2025012503) {
        $table = new xmldb_table('block_zoomonline_links');

        // Define fields for the table.
        $fields = [
                new xmldb_field('id', XMLDB_TYPE_INTEGER, '10', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null, null),
                new xmldb_field('zoom_meeting_id', XMLDB_TYPE_CHAR, '45', null, null, null, null, null),
                new xmldb_field('lecturer_id', XMLDB_TYPE_CHAR, '45', null, null, null, null, null),
                new xmldb_field('course_id', XMLDB_TYPE_CHAR, '45', null, null, null, null, null),
                new xmldb_field('crscode', XMLDB_TYPE_CHAR, '90', null, null, null, null, null),
                new xmldb_field('groupid', XMLDB_TYPE_CHAR, '90', null, null, null, null, null),
                new xmldb_field('visible', XMLDB_TYPE_INTEGER, '1', null, XMLDB_NOTNULL, null, '1', null, 'groupid'),
        ];

        // Define the primary key.
        $key = new xmldb_key('id', XMLDB_KEY_PRIMARY, ['id']);

        // Create the table if it doesn't exist.
        if (!$dbman->table_exists($table)) {
            $table->addFields($fields);
            $table->addKey($key);
            $dbman->create_table($table);
        } else {
            // Ensure all fields exist.
            foreach ($fields as $field) {
                if (!$dbman->field_exists($table, $field)) {
                    $dbman->add_field($table, $field);
                }
            }
        }

        // Savepoint for this upgrade step.
        upgrade_block_savepoint(true, 2025012503, 'zoomonline');
    }

    // Ensure the block_zoomonline_att_check table exists and is correct.
    if ($oldversion < 2025012503) {
        $table = new xmldb_table('block_zoomonline_att_check');

        // Define fields for the table.
        $fields = [
                new xmldb_field('id', XMLDB_TYPE_INTEGER, '10', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null, null),
                new xmldb_field('zoom_meeting_id', XMLDB_TYPE_CHAR, '45', null, null, null, null, null),
                new xmldb_field('start_date', XMLDB_TYPE_DATETIME, null, null, null, null, null, null),
        ];

        // Define the primary key.
        $key = new xmldb_key('id', XMLDB_KEY_PRIMARY, ['id']);

        // Create the table if it doesn't exist.
        if (!$dbman->table_exists($table)) {
            $table->addFields($fields);
            $table->addKey($key);
            $dbman->create_table($table);
        } else {
            // Ensure all fields exist.
            foreach ($fields as $field) {
                if (!$dbman->field_exists($table, $field)) {
                    $dbman->add_field($table, $field);
                }
            }
        }

        // Savepoint for this upgrade step.
        upgrade_block_savepoint(true, 2025012503, 'zoomonline');
    }

    // Ensure the block_zoomonline_token table exists and is correct.
    if ($oldversion < 2025012503) {
        $table = new xmldb_table('block_zoomonline_token');

        // Define fields for the table.
        $fields = [
                new xmldb_field('id', XMLDB_TYPE_INTEGER, '10', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null, null),
                new xmldb_field('access_token', XMLDB_TYPE_CHAR, '1000', null, null, null, null, null),
                new xmldb_field('current', XMLDB_TYPE_INTEGER, '11', null, null, null, null, null),
        ];

        // Define the primary key.
        $key = new xmldb_key('id', XMLDB_KEY_PRIMARY, ['id']);

        // Create the table if it doesn't exist.
        if (!$dbman->table_exists($table)) {
            $table->addFields($fields);
            $table->addKey($key);
            $dbman->create_table($table);
        } else {
            // Ensure all fields exist.
            foreach ($fields as $field) {
                if (!$dbman->field_exists($table, $field)) {
                    $dbman->add_field($table, $field);
                }
            }
        }

        // Savepoint for this upgrade step.
        upgrade_block_savepoint(true, 2025012503, 'zoomonline');
    }

    return true;
}
